#!/usr/bin/pyton
import sys,os,os.path,traceback

# INHX32 format
# :BBAAAATTHHHH...HHHHCC
# BB: two-digit hex byte count
# AAAA: four-digit hex byte starting address (hi,lo)
# TT: two-digit record type:
#     00 = data records
#     01 = end-of-file records
#     04 = extended address record
# HHHH: four-digit hex data (lo,hi)
# CC: two-digit two's complement byte checksum

# reads in a INHX32 format file, returns list
# of ints that are contents of 16-bit words
def read_inhx32(fname):
    result = []
    addr_ext = 0
    try:
        f = open(fname,'r')
        for line in f.xreadlines():
            bb = int(line[1:3],16)     # byte count for data
            aaaa = int(line[3:7],16)   # address
            tt = int(line[7:9],16)    # record type
            words = []                 # read in 16-bit data
            index = 9
            for i in xrange(bb/2):
                lo = int(line[index:index+2],16)
                hi = int(line[index+2:index+4],16)
                words.append(hi<<8 | lo);
                index += 4
            # dispatch on record type
            if tt == 0:         # data
                addr = (addr_ext << 16) + (aaaa & 0xFFFF)
                waddr = addr >> 1
                while waddr > len(result): result.append(0)
                for w in words:
                    if waddr == len(result): result.append(w)
                    else: result[waddr] = w
                    waddr += 1
            elif tt == 1:       # end of file
                break
            elif tt == 4:       # extended address
                addr_ext = words[0]
        f.close()
    except Exception,e:
        print "Oops: in read_inhx32:",e
        sys.exit(0)
    return result

# get name of code/module
if (len(sys.argv) != 2):
    print "Usage: bpicmem <modulename>"
    sys.exit(0)

mname = sys.argv[1]

# read in memory contents
fname = mname + ".hex"
if not os.path.exists(fname):
    print "Oops: can't find %s" % coename
    sys.exit(0)

locations = read_inhx32(fname)
nlocs = len(locations)

# helper function returns binary string with WIDTH
# digits from BITOFFSET within location LOCN
def bits(width,bitoffset,locn):
    if locn >= nlocs: v = 0
    else: v = locations[locn]
    v >>= bitoffset;
    result = []
    for i in xrange(width):
        if v % 2 == 0: result.append('0')
        else: result.append('1')
        v >>= 1
    result.reverse()
    return ''.join(result)

if (nlocs <= 1024) :
    nmems = 1            # use one 1024 x 16 BRAM
    bram = "RAMB16_S18"
    naddr = 10
    width = 16
elif (nlocs <= 2048):
    nmems = 2            # use two 2048 x 8 BRAMs
    bram = "RAMB16_S9"
    naddr = 11
    width = 8
else:
    print "Oops: %d is too big, can only support up to 2k locations" % nlocs
    sys.exit(0)

# ready to create appropriate Verilog module
try:
    vname = mname + ".v"
    v = open(vname,'w')

    # output standard module prologue
    v.write("""// single-port read-only memory initialized with %s code
module %s(addr,clk,dout);
  input [10:0] addr;     // up to 2K locations
  input clk;             // memory has internal address regs
  output [11:0] dout;    // written at rising clock edge

  // we're using %d out of %d locations
  wire [15:0] do;
""" % (mname,mname,nlocs,1 << naddr))

    # output appropriate number of BRAM instances
    for i in xrange(nmems):
        lo = i * width
        hi = lo + width - 1
        v.write("  %s m%d(.CLK(clk),.ADDR(addr[%d:0]) ,.DO(do[%d:%d]),.WE(1'b0),.EN(1'b1),.SSR(1'b0));\n" % (bram,i,naddr-1,hi,lo))
        # output defparams to initialize this BRAM block
        nwords = 256/width
	for init in xrange(64):
	    v.write("  defparam m%d.INIT_%02X = 256'b" % (i,init))
            start = init * nwords
            first = True
            for locn in xrange(start+nwords,start,-1):
                if first: first = False
                else: v.write('_')
                v.write(bits(width,lo,locn-1))
            v.write(';\n')

    v.write("\n  assign dout = do[11:0];\nendmodule")
    v.close()
except Exception,e:
    print "Oops:",e
    sys.exit(0)

# finished!
