// show off pic processor (cjt 10/05)

module pictest(
     // clocks
     clock_50MHz, clock_socket,

     // 256k x 32 SRAM
     sram_a,sram_ce_b,sram_we_b,
     sram_io1,sram_ce1,sram_ub1,sram_lb1,
     sram_io2,sram_ce2,sram_ub2,sram_lb2,

     // I/O
     segment,an,sw,btn,led,
     vga_r,vga_g,vga_b,vga_hs,vga_vs,
     ps2_d,ps2_c,
     rs232_rxd,rs232_txd,rs232_rxd_a,rs232_txd_a,
     flash_d0,flash_oe,flash_rclk,

     // expansion connectors
     A1_4,A1_21,A1_22,
     A2_io,A2_db,A2_astb,A2_dstb,A2_write,A2_wait,A2_reset,
     B1_db,B1_adr,B1_we,B1_oe,B1_cs,B1_astb,B1_dstb,B1_write,B1_wait,B1_reset,B1_int
     );

   input clock_50MHz;
   input clock_socket;

   output [17:0] sram_a;
   output 	 sram_ce_b;
   output 	 sram_we_b;
   inout [15:0]  sram_io1;
   output 	 sram_ce1;
   output 	 sram_ub1;
   output 	 sram_lb1;
   inout [15:0]  sram_io2;
   output 	 sram_ce2;
   output 	 sram_ub2;
   output 	 sram_lb2;

   output [7:0]  segment;   // active low
   output [3:0]  an;        // active low
   input [7:0] 	 sw;
   input [3:0] 	 btn;
   output [7:0]  led;

   output 	 vga_r;
   output 	 vga_g;
   output 	 vga_b;
   output 	 vga_hs;
   output 	 vga_vs;

   inout 	 ps2_d;
   input 	 ps2_c;

   input 	 rs232_rxd;
   output 	 rs232_txd;
   input 	 rs232_rxd_a;
   output 	 rs232_txd_a;

   input 	 flash_d0;
   output 	 flash_oe;
   output 	 flash_rclk;

   inout 	 A1_4,A1_21,A1_22;

   inout [18:1]  A2_io;
   inout [7:0] 	 A2_db;
   inout 	 A2_astb;
   inout 	 A2_dstb;
   inout 	 A2_write;
   inout 	 A2_wait;
   inout 	 A2_reset;

   inout [7:0] 	 B1_db;
   inout [5:0] 	 B1_adr;
   inout 	 B1_we;
   inout 	 B1_oe;
   inout 	 B1_cs;
   inout 	 B1_astb;
   inout 	 B1_dstb;
   inout 	 B1_write;
   inout 	 B1_wait;
   inout 	 B1_reset;
   inout 	 B1_int;

   // SRAM pins
   assign sram_a = 18'h0;
   assign sram_ce_b = 1'b1;
   assign sram_we_b = 1'b1;
   assign sram_io1 = 18'h0;
   assign sram_ce1 = 1'b0;
   assign sram_ub1 = 1'b0;
   assign sram_lb1 = 1'b0;
   assign sram_io2 = 1'b0;
   assign sram_ce2 = 1'b0;
   assign sram_ub2 = 1'b0;
   assign sram_lb2 = 1'b0;

   // misc. I/O
   //assign segment = 8'hFF;   // active low
   //assign an = 4'hF;         // active low
   assign led = 8'hAA;       // show it's us!!!
   assign vga_r = 1'b0;
   assign vga_g = 1'b0;
   assign vga_b = 1'b0;
   assign vga_hs = 1'b0;
   assign vga_vs = 1'b0;
   assign ps2_d = 1'bz;
   assign rs232_txd = 1'b0;
   assign rs232_txd_a = 1'b0;
   assign flash_oe = 1'b1;
   assign flash_rclk = 1'b0;

   // expansion connectors
   assign A1_4 = 1'b0;
   assign A1_21 = 1'b0;
   assign A1_22 = 1'b0;
   assign A2_io = 18'h00000;
   assign A2_db = 8'h00;
   assign A2_astb = 1'b0;
   assign A2_dstb = 1'b0;
   assign A2_write = 1'b0;
   assign A2_wait = 1'b0;
   assign A2_reset = 1'b0;
   assign B1_db = 8'h00;
   assign B1_adr = 6'h00;
   assign B1_we = 1'b0;
   assign B1_oe = 1'b0;
   assign B1_cs = 1'b0;
   assign B1_astb = 1'b0;
   assign B1_dstb = 1'b0;
   assign B1_write = 1'b0;
   assign B1_wait = 1'b0;
   assign B1_reset = 1'b0;
   assign B1_int = 1'b0;

   // produce a 40MHz clock
   wire   clk_unbuf,clk;
   DCM clk1(.CLKIN(clock_50MHz),.CLKFX(clk_unbuf));
   // synthesis attribute CLKFX_DIVIDE of clk1 is 5
   // synthesis attribute CLKFX_MULTIPLY of clk1 is 4
   // synthesis attribute CLK_FEEDBACK of clk1 is NONE
   // synthesis attribute CLKIN_PERIOD of clk1 is 20
   BUFG clk2(.O(clk),.I(clk_unbuf));

   // synchronize external reset (use BTN[3])
   wire reset;
   debounce dbreset(clk,btn[3],reset);

   // PIC core
   wire [9:0] pc;
   wire [11:0] idata;
   wire [3:0] porta;
   wire [7:0] portb;
   pic16c5x pic(.clk(clk),.reset(reset),
		.pc(pc),.idata(idata), // instruction memory
		.port_a_out(porta),    // 7-seg display anodes (active low)
		.port_b_out(portb),    // 7-seg display segments (active low)
		.port_c_in(sw)         // slide switches
		);
   defparam pic.PCMSB = 9;   // 1024 inst locations 

   // PIC program memory (1024 x 12)
   test pgm({1'b0,pc},~clk,idata);        // initialized with rom.coe

   // show memory address in the 7-segments display
   wire clk_300Hz;
   wire [3:0] anodes;
   wire [7:0] segs;
   segdisplay lights(clk,1'b1,{6'b0,pc},anodes,segs,clk_300Hz);

   assign an = btn[0] ? anodes : porta;
   assign segment = btn[0] ? segs : portb;

endmodule // pictest
